using System;
using System.IO;

public class ByteConversionExample {

    // Erstellen eines Byte-Arrays aus dem Dezimal-Wert
    public static byte[] DecimalToByteArray (decimal src) {

        // Erstellen eines MemoryStream-Objektes, das als Puffer
        // verwendet wird, um die binren Daten aufzunehmen
        using (MemoryStream stream = new MemoryStream()) {

            // Erstellen eines BinaryWriter-Objektes zum 
            // Schreiben von binren Daten in den Stream 
            using (BinaryWriter writer = new BinaryWriter(stream)) {

                // Schreiben des Decimal-Wertes in den BinaryWriter
                // und dadurch in den MemoryStream
                writer.Write(src);

                // Zurckgeben der Byte-Darstellung des Decimal-Wertes
                return stream.ToArray();
            }
        }
    }

    // Erstellen eines Decimal-Wertes aus dem Byte-Array
    public static decimal ByteArrayToDecimal (byte[] src) {

        // Erstellen eines MemoryStream-Objektes, 
        // das das Byte-Array umfasst
        using (MemoryStream stream = new MemoryStream(src)) {

            // Erstellen eines BinaryReader-Objektes, zum Auslesen
            // des Decimal-Wertes aus dem Stream
            using (BinaryReader reader = new BinaryReader(stream)) {

                // Lesen und zurckgeben des Decimal-Wertes
                // aus dem BinaryReader/MemoryStream
                return reader.ReadDecimal();
            }
        }
    }

    public static void Main() {

        byte[] b = null;

        // Konvertieren eines bool-Wertes in Byte-Array 
        // und Anzeigen der Ergebnisse
        b = BitConverter.GetBytes(true);
        Console.WriteLine(BitConverter.ToString(b));

        // Konvertieren eines Byte-Arrays in ein bool-Wert
        // und Anzeigen der Ergebnisse
        Console.WriteLine(BitConverter.ToBoolean(b,0));

        // Konvertieren eines int in ein byte-Array
        // und Anzeigen der Ergebnisse
        b = BitConverter.GetBytes(3678);
        Console.WriteLine(BitConverter.ToString(b));

        // Konvertieren eines Byte-Arrays in ein int-Wert
        // und Anzeigen der Ergebnisse
        Console.WriteLine(BitConverter.ToInt32(b,0));

        // Konvertieren eines decimal-Wertes in ein 
        // Byte-Array und Anzeigen der Ergebnisse
        b = DecimalToByteArray(285998345545.563846696m);
        Console.WriteLine(BitConverter.ToString(b));

        // Konvertieren eines Byte-Arrays in ein
        // decimal-Wert und Anzeigen der Ergebnisse
        Console.WriteLine(ByteArrayToDecimal(b));

		// Auf Tastendruck warten...
		Console.ReadLine();
    }
}
