using System;
using System.IO;
using System.Text;

public class Base64Example {

    // Erstellen eines Byte-Arrays aus einem Dezimal-Wert
    public static byte[] DecimalToByteArray (decimal src) {

        // Erstellen eines MemoryStream-Objektes, das als Puffer
        // verwendet wird, um die binren Daten aufzunehmen
        using (MemoryStream stream = new MemoryStream()) {

            // Erstellen eines BinaryWriter-Objektes zum 
            // Schreiben von binren Daten in den Stream 
            using (BinaryWriter writer = new BinaryWriter(stream)) {

                // Schreiben des Decimal-Wertes in den BinaryWriter
                // und dadurch in den MemoryStream
                writer.Write(src);

                // Zurckgeben der Byte-Darstellung des Decimal-Wertes
                return stream.ToArray();
            }
        }
    }

    // Erstellen eines Decimal-Wertes aus dem Byte-Array
    public static decimal ByteArrayToDecimal (byte[] src) {

        // Erstellen eines MemoryStream-Objektes, 
        // das das Byte-Array umfasst
        using (MemoryStream stream = new MemoryStream(src)) {

            // Erstellen eines BinaryReader-Objektes, zum Auslesen
            // des Decimal-Wertes aus dem Stream
            using (BinaryReader reader = new BinaryReader(stream)) {

                // Lesen und zurckgeben des Decimal-Wertes
                // aus dem BinaryReader/MemoryStream
                return reader.ReadDecimal();
            }
        }
    }

// Base64-Kodieren eines Unicode-Strings
public static string StringToBase64 (string src) {

    // Abfragen einer Byte-Darstellung des Quellstrings
    byte[] b = Encoding.Unicode.GetBytes(src);

    // Zurckgeben des Base64-kodieren Strings
    return Convert.ToBase64String(b);
}

// Dekodieren eines Base64-kodierten Unicode-Strings
public static string Base64ToString (string src) {

    // Dekodieren des Base64-kodierten Strings in ein Byte-Array
    byte[] b = Convert.FromBase64String(src);

    // Zurckgeben des dekodierten Unicode-Strings
    return Encoding.Unicode.GetString(b);
}

// Base64-Kodieren eines decimal-Wertes
public static string DecimalToBase64 (decimal src) {

    // Abrufen der Byte-Darstellung des Dezimal-Wertes
    byte[] b = DecimalToByteArray(src);

    // Zurckgeben des Base64-kodierten Dezimal-Wertes
    return Convert.ToBase64String(b);
}

// Dekodieren eines Base64-kodierten Dezimal-Wertes
public static decimal Base64ToDecimal (string src) {

    // Dekodieren des Base64-kodierten decimal-Wertes in ein Byte-Array
    byte[] b = Convert.FromBase64String(src);

    // Zurckgeben des dekodierten decimal-Wertes
    return ByteArrayToDecimal(b);
}

// Base64-kodieren eines int-Wertes
public static string IntToBase64 (int src) {

    // Abrufen der Byte-Darstellung des int-Wertes
    byte[] b = BitConverter.GetBytes(src);

    // Zurckgeben des Base64-kodierten int-Wertes
    return Convert.ToBase64String(b);
}

// Einen Base64-kodierten int dekodieren
public static int Base64ToInt (string src) {

    // Dekodieren des Base64-kodierten int-Wertes in ein Byte-Array
    byte[] b = Convert.FromBase64String(src);

    // Zurckgeben des dekodierten int-Wertes
    return BitConverter.ToInt32(b,0);
}

    public static void Main() {

        // Kodieren und Dekodieren der Zeichenkette
        Console.WriteLine(StringToBase64
            ("Welcome to the C# Programmer's Cookbook"));
        Console.WriteLine(Base64ToString("VwBlAGwAYwBvAG0AZQAgAHQAbwA" + 
            "gAHQAaABlACAAQwAjACAAUAByAG8AZwByAGEAbQBtAGUAcgAnAHMAIAB" +
            "DAG8AbwBrAGIAbwBvAGsA"));

        // Kodieren und Dekodieren eines decimal-Wertes
        Console.WriteLine(DecimalToBase64(285998345545.563846696m));
        Console.WriteLine(Base64ToDecimal("KDjBUP07BoEPAAAAAAAJAA=="));

        // Kodieren und Dekodieren eines int-Wertes
        Console.WriteLine(IntToBase64(35789));
        Console.WriteLine(Base64ToInt("zYsAAA=="));

		// Auf Tastendruck warten...
		Console.Read();
    }
}
