using System;
using System.Reflection;

// Ein allgemeines Interface, dass all Plug-Ins implementieren mssen.
public interface IPlugin {
    string Beschreibung { get; set; }
    void Start();
    void Stop();
}

// Eine abstrake Basisklasse, von der alle Plug-Ins abstammen mssen.
public abstract class AbstraktesPlugin : IPlugin {

    // Festhalten einer Beschreibung fr die Plug-In-Instanz.
    private string beschreibung = "";

    // Abgeschlossene Eigenschaft, um die Plug-In-Beschreibung auszulesen.
    public string beschreibung { 
        get { return beschreibung; }
        set { beschreibung = value; }
    }

    // Member des IPlugin-Interface als abstrakt deklarieren.
    public abstract void Start();
    public abstract void Stop();
}

// Eine simple IPlugin-Implementierung, um die PluginFactory-Klasse zu demonstrieren.
public class SimplesPlugin : AbstraktesPlugin {

    // Implementieren der Start-Methode.
    public override void Start() {
        Console.WriteLine(Beschreibung  + ": Startet...");
    }

    // Implementieren der Stop-Methode.
    public override void Stop() {
        Console.WriteLine(Beschreibung + ": Stoppt...");
    }
}

// Eine Factory zum instanziieren von IPlugin-Instanzen.
public sealed class PluginFactory {

    public static IPlugin CreatePlugin(string assembly, 
        string pluginName, string Beschreibung) {

        // Beschaffen des Typs fr das spezifizierte Plug-In.
        Type type = Type.GetType(pluginName + ", " + assembly);

        // Beschaffen des ConstructorInfo-Objekts.
        ConstructorInfo cInfo = type.GetConstructor(Type.EmptyTypes);

        // Erstellen des Objekts und seine Konvertierung in den Typ StringBuilder.
        IPlugin plugin = (IPlugin)cInfo.Invoke(null);

        // Konfiguration des neuen IPlugin.
        plugin.Beschreibung = beschreibung;

        return plugin;
    }
}