using System;
using System.Threading;

// Eine Klasse, die verwendet wird, um Daten an DisplayMessage zu bergeben,
// wenn sie durch den Thread-Pool ausgefhrt wird.
public class MessageInfo {

    private int iterations;
    private string message;

    // Konstruktor, der die Konfigurationseinstellungen fr den Thread bernimmt.
    public MessageInfo(int iterations, string message) {

        this.iterations = iterations;
        this.message = message;
    }

    // Eigenschaften, um die Konfigurationseinstellungen zu ermitteln.
    public int Iterations { get { return iterations; } }
    public string Message { get { return message; } }
}

public class ThreadPoolExample {

    // Eine Nachricht im Konsolenfenster ausgeben.
    public static void DisplayMessage(object state) {

        // Statusparamter in ein MessageInfo-Objekt casten.
        MessageInfo config = state as MessageInfo;

        // Falls der Konfigurationsparameter null ist, wurden der 
		// ThreadPool.QueueUserWorkItem-Methode keine Parameter bergeben;
		// dann die Standardwerte verwenden.
        if (config == null) {

            // Eine vordefinierte Nachricht dreimal im Konsolenfenster ausgeben.
            for (int count = 0; count < 3; count++) {

                Console.WriteLine("Ein Thread-Pool-Beispiel.");

                // Sleep nur fr Demonstrationszwecke. Vermeiden Sie das Aussetzen des Threads
				// mit Sleep in echten Anwendungen!
                Thread.Sleep(1000);
            }

        } else {

            // Die angegebene Nachricht entsprechend oft ausgeben.
            for (int count = 0; count < config.Iterations; count++) {

                Console.WriteLine(config.Message);

				// Sleep nur fr Demonstrationszwecke. Vermeiden Sie das Aussetzen des Threads
				// mit Sleep in echten Anwendungen!
				Thread.Sleep(1000);
            }
        }
    }

    public static void Main() {

        // Erstellen einer Delegaten-Instanz, um zu erlauben, die DisplayMessage-Methode
		// dem Thread-Pool zwecks Ausfhrung hinzuzufgen.
        WaitCallback workMethod = 
            new WaitCallback(ThreadPoolExample.DisplayMessage);

        // Ausfhren von DisplayMessage unter Verwendung des Thread-Pools und ohne Paramerter.
        ThreadPool.QueueUserWorkItem(workMethod);

        // DisplayMessage unter Verwendung des Thread-Pools und mit der Mglichkeit
		// zur bergabe eines MessageInfo-Objektes, das DisplayMessage bergeben wird, ausfhren.
        MessageInfo info = 
            new MessageInfo(5, "Ein Thread-Pool-Beispiel mit Parametern.");

        ThreadPool.QueueUserWorkItem(workMethod, info);

        // Auf weitere Ausfhrung warten.
        Console.WriteLine("Hauptmethode beendet. Drcken Sie Return zum Beenden.");
        Console.ReadLine();
    }
}