using System;
using System.Threading;

public class RunAtExample {

	// Die Methode, die ausgefhrt wird, wenn der Timer abluft. Zeigt eine
	// Nachricht im Konsolenfenster an.
    private static void TimerHandler(object state) {

        Console.WriteLine("{0} : Timer abgelaufen",
            DateTime.Now.ToString("HH:mm:ss.ffff"));
    }

    public static void RunAt(DateTime execTime) {

        // Errechnen des Unterschieds zwischen dem gewnschten Ausfhrungszeitpunktes
		// und der aktuellen Uhrzeit.
        TimeSpan waitTime = execTime - DateTime.Now;

        if (waitTime < new TimeSpan(0)) waitTime = new TimeSpan(0);

        // Erstellen einer neuen TimerCallback-Delegateninstanz, 
		// die eine statische TimerHandler-Methode referentiert.
		// TimerHandler wird aufgerufen, wenn der Timer abluft.
        TimerCallback handler = new TimerCallback(TimerHandler);

        // Einen Timer erstellen, der einmal zu einem bestimmten Zeitpunkt auslst.
		// Geben Sie -1 als Intervallwert ein, um das wiederholte Auslsen des
		// Timers zu unterbinden.
        new Timer(handler, null, waitTime, new TimeSpan(-1));
    }

    public static void Main(string[] args) {

        DateTime execTime;

        // Sicherstellen, dass ein Ausfhrungszeitpunkt ber die Kommandozeile bestimmt wurde.
        if (args.Length > 0) {

            // Den String ins DateTime-Format umwandeln. Nur RFC1123-Zeitformat wird unterstzt.
            try {

                execTime = DateTime.ParseExact(args[0],"r", null);

                Console.WriteLine("Aktueller Zeitpunkt   : " +
                    DateTime.Now.ToString("r"));

                Console.WriteLine("Ausfhrungszeitpunkt  : " +
                    execTime.ToString("r"));

                RunAt(execTime);

            } catch (FormatException) {
 
                Console.WriteLine("Ausfhrungszeitpunkt muss im folgenden Format eingegeben werden:" +
                    " ddd, dd MMM yyyy HH':'mm':'ss 'GMT'");
            }

            // Wait to continue.
            Console.WriteLine("Hauptmethode abgeschlossen. Drcken Sie Return.");
            Console.ReadLine();

        } else {

            Console.WriteLine("Geben Sie den Zeitpunkt an, zu dem Sie" +
                " die Ausfhrung starten wollen (im folgenden Format) : " + 
                " ddd, dd MMM yyyy HH':'mm':'ss 'GMT'");
        }
    }
}