using System;
using System.Threading;

public class TimerExample {

    // Die Methode, die ausgefhrt wird, sobald der Timer abgelaufen ist.
	// Zeigt eine Nachricht im Konsolenfenster an.
    private static void TimerHandler(object state) {

        Console.WriteLine("{0} : {1}",
            DateTime.Now.ToString("HH:mm:ss.ffff"), state);
    }

    public static void Main() {

        // Erstellt eine neue TimeCallback-Delegateninstanz, die
		// die statische TimerHandler-Methode referenziert.
		// TimerHandler wird aufgerufen, wenn der Timer abluft.
        TimerCallback handler = new TimerCallback(TimerHandler);

        // Erstellt ein Zustandsobjekt, das der TimerHandler-Methode
		// bergeben wird, wenn sie ausgelst wird. In diesem Fall eine
		// darzustellende Nachricht.
        string state = "Timer abgelaufen.";

        Console.WriteLine("{0} : Erstelle Timer.",
            DateTime.Now.ToString("HH:mm:ss.ffff"));

        // Erstellen eines Timers der das erste Mal nach zwei Sekunden und dann
		// jede Sekunde auslst.
        using (Timer timer = new Timer(handler, state, 2000, 1000)) {

            int period;

            // Das neue Timer-Intervall aus dem Konsolenfenster
			// lesen, solange bis der Anwender den Wert 0 eingegeben hat.
			// Ungltige Werte bedeuten das gleiche wie 0 und beenden das Beispiel.
            do {

                try {
                    period = Int32.Parse(Console.ReadLine());
                } catch {
                    period = 0;
                }

                // Den Timer so umprogrammieren, dass der neue Intervallwert bercksichtigt wird;
				// Das neue Intervall beginnt unmittelbar.
                if (period > 0) timer.Change(0, period);

            } while (period > 0);
        }

        // Wait to continue.
        Console.WriteLine("Hauptmethode abgeschlossen. Drcken Sie Return.");
        Console.ReadLine();
    }
}
