using System;
using System.Threading;

public class EventExecutionExample {

    // Eine Methode, die ausgefhrt wird, wenn das AutoResetEvent-Objekt
	// zum Zustand "signalisiert" wechselt oder wenn die Warteroutine
	// die Timeout-Schwelle erreicht hat.
    private static void EventHandler(object state, bool timedout) {

        // Eine entsprechende Nachricht im Konsolenfenster ausgeben,
		// basierend auf dem Ereignis - Timeout beim Warten oder signalisiertes 
		// AutoResetEvent.
        if (timedout) {
            
            Console.WriteLine("{0} : Bis zum Timeout gewartet.",
                DateTime.Now.ToString("HH:mm:ss.ffff"));
        } else {

            Console.WriteLine("{0} : {1}", 
                DateTime.Now.ToString("HH:mm:ss.ffff"), state);
        }
    }

    public static void Main() {

        // Ein neues AutoResetEvent-Objekt im unsignalisierten Zustand erstellen.
        AutoResetEvent autoEvent = new AutoResetEvent(false);

        // Erstellen einer WaitOrTimerCallback-Delegateninstanz, die
		// die statische Methode EventHandler referenziert. EventHandler
		// wird aufgerufen, wenn das AutoResetEvent-Objekt in den signalisierten
		// Zustand wechselt oder das Warten die Timeout-Zeitspanne berschritten hat.
        WaitOrTimerCallback handler = 
            new WaitOrTimerCallback(EventHandler);

        // Erstellen des Zustandobjektes, dass der Ereignisbehandlungsroutine 
		// bergeben wird, wenn die Methode ausgelst wird. In diesem
		// Fall eine anzuzeigende Nachricht.
        string state = "AutoResetEvent im signalisierten Zustand.";

        // Die Delegateninstanz so konfigurieren, dass sie wartet, 
		// bis AutoResetEvent in den signalisierten Zustand wechselt.
		// Ein Timeout von 3 Sekunden definieren, und die Warteroutine
		// so konfigurieren, dass sie sich automatisch in den Ausgangszustand zurckversetzt.
		// (letzter Partameter).
        RegisteredWaitHandle handle = 
            ThreadPool.RegisterWaitForSingleObject(autoEvent, handler, 
            state, 3000, false);

		Console.WriteLine("Drcken Sie Return, um AutoResetEvent in den signalisierten Zustand zu versetzen,\n" +
			" oder geben Sie \"Abbruch\" ein, um die Warteroutine zu deregistrieren (beenden).");

        while (Console.ReadLine().ToUpper() != "ABBRUCH") {

            // Falls "Abbruch" nicht ber die Konsole eingegeben wurde,
			// AutoResetEvent in den signalisierten Zustand versetzen, was dazu fhrt,
			// dass EventHandler ausgefhrt wird. Das AutoResetEvent-Objekt
			// versetzt sich danach automatisch wieder in den unsignalisierten Zustand.
            autoEvent.Set();
        }

        // Die WarteRoutine deregistrieren.
        Console.WriteLine("Deregistrieren der Warteroutine.");
        handle.Unregister(null);

        // Auf Programmende warten.
        Console.WriteLine("Hauptroutine abgeschlossen. Drcken Sie Return.");
        Console.ReadLine();
    }
}
