using System;
using System.Threading;

public class ThreadControlExample {

    private static void DisplayMessage() {

        // Nachricht wiederholt im Konsolenfenster anzeigen.
        while (true) {

            try {

                Console.WriteLine("{0} : Zweiter Thread luft. Geben Sie"
                    + " (A)aussetzen, (W)iederaufnehmen, (U)nterbrechen, or (B)eenden ein.",
                    DateTime.Now.ToString("HH:mm:ss.ffff"));

                // Fr 2 Sekunden aussetzen.
                Thread.Sleep(2000);

            } catch (ThreadInterruptedException) {

                // Der Thread wurde unterbrochen. Die ThreadInterruptedException
				// erlaubt es dem Beispiel die erforderlichen Schritte zu unternehmen
				// und die Ausfhrung fortzusetzen.
                Console.WriteLine("{0} : Zweiter Thread wurde unterbrochen.",
                    DateTime.Now.ToString("HH:mm:ss.ffff"));

            } catch (ThreadAbortException abortEx) {

                // Das Objekt in der ThreadAbortException.ExceptionState-Eigenschaft
				// wird durch den Thread zur Verfgun gestellt, der Thread.Abort aufgerufen hat.
				// In diesem Fall enthlt es den Text fr den Abbruchgrund.
                Console.WriteLine("{0} : Zweiter Thread abgebrochen ({1})",  
                    DateTime.Now.ToString("HH:mm:ss.ffff"),
                    abortEx.ExceptionState);

                // Obwohl ThreadAbortException behandelt wurde, lst die Runtime
				// die Ausnahme ein weiteres Mal aus, um sicherzustellen, dass
				// der Thread abgeschlossen wird.
            }
        }
    }

    public static void Main() {

        // Ein Thread-Objekt erstellen und ihn der ThreadStart-Delegateninstanz 
		// bergeben, die DisplayMessage referenziert.
        Thread thread = new Thread(new ThreadStart(DisplayMessage));

        Console.WriteLine("{0} : Zweiten Thread starten.",  
            DateTime.Now.ToString("HH:mm:ss.ffff"));

        // Zweiten Thread starten.
        thread.Start();

        // Durch den Anwender eingegebene Befehle in einer Schleife verarbeiten.
        char command = ' ';

        do {

            string input = Console.ReadLine();
            if (input.Length > 0) command = input.ToUpper()[0];
            else command = ' ';

            switch (command) {

                case 'A':
                    // Zweiten Thread aussetzen (Suspend)
                    Console.WriteLine("{0} : Aussetzen des zweiten Threads.",
                        DateTime.Now.ToString("HH:mm:ss.ffff"));
                    thread.Suspend();
                    break;

                case 'W':
                    // Zweiten Thread wieder aufnehmen.
                    try { 
                        Console.WriteLine("{0} : Zweiten Thread wieder aufnehmen.",
                            DateTime.Now.ToString("HH:mm:ss.ffff"));
                        thread.Resume();
                    } catch (ThreadStateException) {
                        Console.WriteLine("{0} : Der Thread konnte nicht ausgesetzt werden.",
                            DateTime.Now.ToString("HH:mm:ss.ffff"));
                    }
                    break;

                case 'U':
                    // Den zweiten Thread unterbrechen.
                    Console.WriteLine("{0} : Zweiten Thread unterbrechen.",
                        DateTime.Now.ToString("HH:mm:ss.ffff"));
                    thread.Interrupt();
                    break;

                case 'B':
					// Zweiten Thread abbrechen und das Zustandsobjekt (in diesem Fall
					// eine Nachricht) dem abzubrechenden Thread bergeben.
                    Console.WriteLine("{0} : Abbrechen des zweiten Threads.",  
                        DateTime.Now.ToString("HH:mm:ss.ffff"));

                    thread.Abort("Beispiel wird beendet.");

                    // Warten, bis der zweite Thread beendet wurde.
                    thread.Join();
                    break;
            }
        } while (command != 'B');

        // Auf Abschluss warten.
        Console.WriteLine("Hauptmethode abgeschlossen. Drcken Sie Return.");
        Console.ReadLine();
    }
}