using System;
using System.Threading;

public class ThreadSyncExample {

    
	// Ein statisches Objekt fr das Locking in statischen Methoden deklarieren,
	// da es fr statische Elemente kein 'this' (Instanzzugriff) gibt.
    private static object consoleGate = new Object();

    private static void DisplayMessage() {

        Console.WriteLine("{0} : Thread starte, ein Objekt fr Thread-Lock erhalten...",
            DateTime.Now.ToString("HH:mm:ss.ffff"));

        // 
		// Einen Lock auf das consoleGate-Objekt erhalten
        try {

            Monitor.Enter(consoleGate);

            Console.WriteLine("{0} : {1}",
                DateTime.Now.ToString("HH:mm:ss.ffff"),
                "consoleGate-Lock erhalten und darauf warten...");

            // Warten, bis Pulse fr das consoleGate-Objekt aufgerufen wurde.
            Monitor.Wait(consoleGate);

            Console.WriteLine("{0} : Thread wurde mit Pulse signalisiert und wird beendet.",
                DateTime.Now.ToString("HH:mm:ss.ffff"));

        } finally {

            Monitor.Exit(consoleGate);
        }
    }

    public static void Main() {

		// Einen Lock auf das consoleGate-Objekt erhalten
		lock (consoleGate) 
		{

            // Erstellen und starten von drei neuen Threads, die
			// DisplayMessage ausfhren.
            for (int count = 0; count < 3; count++) {

                (new Thread(new ThreadStart(DisplayMessage))).Start();
            }
        }

        Thread.Sleep(1000);

        // Einen einzigen, wartenden Thread wieder fortsetzen.
        Console.WriteLine("{0} : {1}", 
            DateTime.Now.ToString("HH:mm:ss.ffff"),
            "Drcken Sie Return, um den wartenden Thread mit Pulse zu signalisieren.");

        Console.ReadLine();

		// Einen Lock auf das consoleGate-Objekt erhalten
		lock (consoleGate) 
		{
            
            // Einen wartenden Thread mit Pulse signalisieren.
            Monitor.Pulse(consoleGate);
        }

        // Alle wartenden Threads fortfhren.
        Console.WriteLine("{0} : {1}",  
            DateTime.Now.ToString("HH:mm:ss.ffff"),
            "Drcken Sie Return, um alle wartenden Threads mit Pulse zu signalisieren.");

        Console.ReadLine();

		// Einen Lock auf das consoleGate-Objekt erhalten
		lock (consoleGate) 
		{
            
            // Alle wartenden Threads mit Pulse zu signalisieren
            Monitor.PulseAll(consoleGate);
        }

        // Auf Programmende warten.
        Console.WriteLine("Hauptmethode abgeschlossen. Drcken Sie Return.");
        Console.ReadLine();
    }
}