using System;
using System.Xml;
using System.Xml.Schema;

public class ValidateXml
{
	[STAThread]
	private static void Main()
	{
		ConsoleValidator consoleValidator = new ConsoleValidator();
		Console.WriteLine("Validating ProductCatalog.xml.");
		
        bool success = consoleValidator.ValidateXml("ProductCatalog.xml", "ProductCatalog.xsd");
		if (!success)
		{
			Console.WriteLine("berprfung fehlgeschlagen.");
		}
		else
		{
			Console.WriteLine("berprfung erfolgreich.");
		}

		Console.ReadLine();
		Console.WriteLine("Validating ProductCatalog_Invalid.xml.");
		success = consoleValidator.ValidateXml("ProductCatalog_Invalid.xml", "ProductCatalog.xsd");
		if (!success)
		{
			Console.WriteLine("berprfung fehlgeschlagen.");
		}
		else
		{
			Console.WriteLine("berprfung erfolgreich.");
		}

		Console.ReadLine();
	}
}

public class ConsoleValidator
{
	// Auf true setzen, wenn mindestens ein Fehler aufgetreten ist.
	private bool failed;

	public bool Failed
	{
		get {return failed;}
	}

	public bool ValidateXml(string xmlFilename, string schemaFilename)
	{
		// Den Validator (berprfer) erstellen.
		XmlTextReader r = new XmlTextReader(xmlFilename);
		XmlValidatingReader validator = new XmlValidatingReader(r);
		validator.ValidationType = ValidationType.Schema;
					
		// Die Schema-Datei in den Validator laden.
		XmlSchemaCollection schemas = new XmlSchemaCollection();
		schemas.Add(null, schemaFilename);
		validator.Schemas.Add(schemas);
		
		// Die Validation-Ereignisbehandlungsprozedur einrichten.
		validator.ValidationEventHandler += new ValidationEventHandler(ValidationEventHandler);
        
		failed = false;
		try
		{
			// Alle XML-Daten lesen.
			while (validator.Read())
			{}
		}
		catch (XmlException err)
		{
			// Tritt auf, wenn das XML-Dokument ungltige Zeichen oder
			// Markierungen enthlt, die nicht sauber verschachtelt oder geschlossen wurden.
			Console.WriteLine("Ein kritischer XML-Fehler ist aufgetreten.");
			Console.WriteLine(err.Message);
			failed = true;
		}
		finally
		{
			validator.Close();
		}

		return !failed;
	}

	private void ValidationEventHandler(object sender, ValidationEventArgs args)
	{
		failed = true;

		// Den berprfungs-Fehler anzeigen.
		Console.WriteLine("Validation error: " + args.Message);
		Console.WriteLine();
	}
}

public class validateXmlDocument
{
	public static void validate()
	{
		XmlDocument doc = new XmlDocument();
		XmlTextReader r = new XmlTextReader("ProductCatalog.xml");
		XmlValidatingReader validator = new XmlValidatingReader(r);
		// Schema in den Validator (berprfer) laden.
		validator.ValidationType = ValidationType.Schema;
		XmlSchemaCollection schemas = new XmlSchemaCollection();
		schemas.Add(null, "ProductCatalog.xsd");
		validator.Schemas.Add(schemas);
		// Dokument laden und gleichzetig validieren.
		// Behandeln Sie nicht das ValidationEventHandler-Ereignis. Stattdessen
		// gestatten Sie es, alle Fehler als XmlSchemaException auslsen zu lassen.
		try 
		{
			doc.Load(validator);
			// (Validierung erfolgreich, wenn das Programm diesen Punkt erreichte.)
		}
		catch (XmlSchemaException err) 
		{
			// (Validierung fehlgeschlagen, wenn das Programm diesen Punkt erreichte.)
		}
	}
}
