using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

public class SerializeXml
{
	private static void Main()
	{
		// Produktkatalog erstellen.
		ProductCatalog catalog = new ProductCatalog("New Catalog", DateTime.Now.AddYears(1));
		Product[] products = new Product[2];
		products[0] = new Product("Product 1", 42.99m);
		products[1] = new Product("Product 2", 202.99m);
		catalog.Products = products;

		// Bestellung in eine Datei serialisieren.
		XmlSerializer serializer = new XmlSerializer(typeof(ProductCatalog));
		FileStream fs = new FileStream("ProductCatalog.xml", FileMode.Create);
		serializer.Serialize(fs, catalog);
		fs.Close();

		catalog = null;

		// Bestellung aus einer Datei deserialisieren.
		fs = new FileStream("ProductCatalog.xml", FileMode.Open);
		catalog = (ProductCatalog)serializer.Deserialize(fs);

		// Bestellung ins Konsolenfenster serialisieren.
		serializer.Serialize(Console.Out, catalog);
		Console.ReadLine();
	}
}


[XmlRoot("productCatalog")]
public class ProductCatalog 
{
	[XmlElement("catalogName")]
	public string CatalogName;

	// Den date-Datentyp verwenden (und den Zeit-Abschnitt im serialisierten XML ignorieren)
	[XmlElement(ElementName="expiryDate", DataType="date")]
	public DateTime ExpiryDate;

	// Namen des Tag festlegen, der alle Produkte speichert,
	// sowie den Namen des product-Tag selbst.
	[XmlArray("products")]
	[XmlArrayItem("product")]
    public Product[] Products;

	public ProductCatalog()
	{
		// Standardkonstruktor fr die Deserialisierung.
	}

	public ProductCatalog(string catalogName, DateTime expiryDate)
	{
		this.CatalogName = catalogName;
		this.ExpiryDate = expiryDate;
	}
}


public class Product 
{
	[XmlElement("productName")]
    public string ProductName;

	[XmlElement("productPrice")]
	public decimal ProductPrice;

	[XmlElement("inStock")]
	public bool InStock;

	[XmlAttributeAttribute(AttributeName="id", DataType="integer")]
	public string Id;

	public Product()
	{
		// Standardkonstruktor fr die Deserialisierung.
	}

	public Product(string productName, decimal productPrice)
	{
		this.ProductName = productName;
		this.ProductPrice = productPrice;
	}
}