using System;
using System.Data;
using System.Data.SqlClient;

public class CommandExample {

    public static void Main() {

        // Ein neues SqlConnection-Objekte erstellen.
        using (SqlConnection con = new SqlConnection()) {

            // Die Verbindungszeichenfolge des SqlConnection-Objekts konfigurieren.
            con.ConnectionString = "Data Source = localhost;" + 
                "Database = Northwind; Integrated Security=SSPI";

	// Die Datenbankverbindung ffnen und die Beispielbefehle
	// ber die Verbindung ausfhren.
            con.Open();

            ExecuteNonQueryExample(con);
            Console.WriteLine(System.Environment.NewLine);

            ExecuteReaderExample(con);
            Console.WriteLine(System.Environment.NewLine);
            
            ExecuteScalarExample(con);
        }

        // Auf Tastendruck warten.
        Console.ReadLine();
    }

    public static void ExecuteNonQueryExample(IDbConnection con) {

        // Einen neuen Befehl erstellen und konfigurieren.
        IDbCommand com = con.CreateCommand();
        com.CommandType = CommandType.Text;
        com.CommandText = "UPDATE Employees SET Title = 'Sales Director'" +
            " WHERE EmployeeId = '5'";

        // Den Befehl ausfhren und das Ergebnis verarbeiten.
        int result = com.ExecuteNonQuery();

        if (result == 1) {
            Console.WriteLine("Titel des Angestellten wurde aktualisiert.");
        } else {
            Console.WriteLine("Titel des Angestellten wurde nicht aktualisiert.");
        }
    }

    public static void ExecuteReaderExample(IDbConnection con) {

        // Einen neuen Befehl erstellen und konfigurieren.
        IDbCommand com = con.CreateCommand();
        com.CommandType = CommandType.StoredProcedure;
        com.CommandText = "Ten Most Expensive Products";

        // Den Befehl ausfhren und das Ergebnis verarbeiten.
        using (IDataReader reader = com.ExecuteReader()) {

            Console.WriteLine("Preise der zehn teuersten Produkte.");

            while (reader.Read()) {

                // Produktdetails anzeigen.
                Console.WriteLine("  {0} = {1}", 
                    reader["TenMostExpensiveProducts"],
                    reader["UnitPrice"]);
            }
        }
    }

    public static void ExecuteScalarExample(IDbConnection con) {

        // Einen neuen Befehl erstellen und konfigurieren.
        IDbCommand com = con.CreateCommand();
        com.CommandType = CommandType.Text;
        com.CommandText = "SELECT COUNT(*) FROM Employees";

        // Den Befehl ausfhren und das Ergebnis konvertieren.
        int result = (int)com.ExecuteScalar();

        Console.WriteLine("Verarbeitete Angestellte = " + result);
    }
}
