using System;
using System.Data;
using System.Data.SqlClient;

public class ParameterExample {

    public static void Main() {

        // Ein neues SqlConnection-Objekte erstellen.
        using (SqlConnection con = new SqlConnection()) {

            // Die Verbindungszeichenfolge des SqlConnection-Objekts konfigurieren.
            con.ConnectionString = "Data Source = localhost;" + 
                "Database = Northwind; Integrated Security=SSPI";

	// Datenbankverbindung ffnen und die Beispielbefehle ber
	// die Verbindung ausfhren.
            con.Open();

            ParameterizedCommandExample(con, "5", "Cleaner");

            StoredProcedureExample(con, "Seafood", "1999");
        }

        // Auf Tastendruck warten.
        Console.ReadLine();
    }

    public static void ParameterizedCommandExample(SqlConnection con, 
        string employeeID, string title) {

        // Einen neuen Befehl mit zwei benannten Parametern erstellen und konfigurieren.
        SqlCommand com = con.CreateCommand();
        com.CommandType = CommandType.Text;
        com.CommandText = "UPDATE Employees SET Title = @title" +
            " WHERE EmployeeId = @id";

        // Ein SqlParameter-Objekte fr den Parameter title erstellen.
        SqlParameter p1 = com.CreateParameter();
        p1.ParameterName = "@title";
        p1.SqlDbType = SqlDbType.VarChar;
        p1.Value = title;
        com.Parameters.Add(p1);

        // Den Parameter id mit einer Kurzformsyntax hinzufgen.
        com.Parameters.Add("@id",SqlDbType.Int).Value = employeeID;

        // Den Befehl ausfhren und das Ergebnis verarbeiten.
        int result = com.ExecuteNonQuery();

        if (result == 1) {
            Console.WriteLine("Der Titel des Angestellten {0} wurde wie folgt aktualisiert: {1}.",
                employeeID, title);
        } else {
            Console.WriteLine("Der Titel des Angestellten {0} wurde nicht aktualisiert.",
                employeeID);
        }
    }

    public static void StoredProcedureExample(SqlConnection con, 
        string category, string year) {

        // Einen neuen Befehl erstellen und konfigurieren.
        SqlCommand com = con.CreateCommand();
        com.CommandType = CommandType.StoredProcedure;
        com.CommandText = "SalesByCategory";

        // Ein SqlParameter-Objekte fr den Parameter category erstellen.
        com.Parameters.Add("@CategoryName",SqlDbType.NVarChar).Value=category;

        // Ein SqlParameter-Objekte fr den Parameter year erstellen.
        com.Parameters.Add("@OrdYear",SqlDbType.NVarChar).Value = year;

        // Den Befehl ausfhren und das Ergebnis verarbeiten.
        using (IDataReader reader = com.ExecuteReader()) {

            Console.WriteLine("Verkufe nach Kategorie ({0}).", year);

            while (reader.Read()) {

                // Produktdetails anzeigen.
                Console.WriteLine("  {0} = {1}", 
                    reader["ProductName"],
                    reader["TotalPurchase"]);
            }
        }
    }
}
