using System;
using System.Net;
using System.Net.Sockets;

public class PingTest
{
	private static void Main()
	{
		Console.WriteLine("Millisekunden fr den Kontakt zu www.yahoo.com: " +
			Pinger.GetPingTime("www.yahoo.com").ToString());
		Console.WriteLine("Millisekunden fr den Kontakt zu  www.seti.org: " +
			Pinger.GetPingTime("www.seti.org").ToString());
		Console.WriteLine("Millisekunden fr den Kontakt zum lokalen Computer: " +
			Pinger.GetPingTime("127.0.0.1").ToString());

		Console.ReadLine();

	}
}


public class Pinger 
{
	const int SOCKET_ERROR = -1;        
	const int ICMP_ECHO = 8;

	public static int GetPingTime(string host)
	{
		int PingTime = 0;

		IPHostEntry serverHE, fromHE;
		int nBytes = 0;
		int dwStart = 0, dwStop = 0;
		IcmpPacket packet = new IcmpPacket();

		if (host == null)
		{
			return -1;
		}
	    
		Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Raw, ProtocolType.Icmp);

		// Den Server-Endpunkt abrufen.
		serverHE = Dns.GetHostByName(host);

		if (serverHE == null) 
		{
			return -1; // fail
		}

		// Den IPEndPoint des Servers in einen EndPoint konvertieren.
		IPEndPoint ipepServer = new IPEndPoint(serverHE.AddressList[0], 0);
		EndPoint epServer = (ipepServer);	

		// Den empfangenen Endpunkt fr den Client-Rechner setzen.
		fromHE = Dns.GetHostByName(Dns.GetHostName());
		IPEndPoint ipEndPointFrom = new IPEndPoint(fromHE.AddressList[0], 0);        
		EndPoint EndPointFrom = (ipEndPointFrom);

		int PacketSize = 0;

		for (int j = 0; j < 1; j++)
		{
			// Das zu sendende Paket erstellen.
			packet.Type = ICMP_ECHO;
			packet.SubCode = 0;
			packet.CheckSum = UInt16.Parse("0");
			packet.Identifier   = UInt16.Parse("45"); 
			packet.SequenceNumber  = UInt16.Parse("0"); 

			int PingData = 32; 
			packet.Data = new Byte[PingData];

			for (int i = 0; i < PingData; i++)
				packet.Data[i] = (byte)'#';

			PacketSize = PingData + 8;

			
			// Sicherstellen, dass das Byte-Array icmp_pkt_buffer eine unvernderliche Gre aufweist.
			if (PacketSize % 2 == 1)
				++PacketSize;

			Byte [] icmp_pkt_buffer = new Byte[ PacketSize ]; 

			int index = 0;          
			index = Serialize(  
				packet, 
				icmp_pkt_buffer, 
				PacketSize, 
				PingData );

			if (index == -1)
				return -1;


			//
			// In ein UInt16-Array konvertieren
			//
	        
			Double double_length = Convert.ToDouble(index);
	    
			Double dtemp = Math.Ceiling( double_length / 2);

			int cksum_buffer_length = Convert.ToInt32(dtemp);
	        
			UInt16 [] cksum_buffer = new UInt16[cksum_buffer_length];

			int icmp_header_buffer_index = 0;

			for( int i = 0; i < cksum_buffer_length; i++ ) 
			{
				cksum_buffer[i] = 
					BitConverter.ToUInt16(icmp_pkt_buffer,icmp_header_buffer_index);
				icmp_header_buffer_index += 2;
			}

	                        
			UInt16 u_cksum = checksum(cksum_buffer, cksum_buffer_length);
			packet.CheckSum  = u_cksum; 
	        
			// Nachdem nun die Prfsumme vorhanden ist, das Paket erneut serialisieren.
			Byte [] sendbuf = new Byte[ PacketSize ]; 
	        
			index = Serialize(  
				packet, 
				sendbuf, 
				PacketSize, 
				PingData );

			if (index == -1)
				return -1;

			dwStart = System.Environment.TickCount; // Start timing

			if ((nBytes = socket.SendTo(sendbuf, PacketSize, 0, epServer)) == SOCKET_ERROR) 
			{

				Console.WriteLine("Error calling sendto");
				return -1; // fail
			}

			// Puffer initialisieren. Der Empfangspuffer hat die Gre des
			// ICMP-Headers plus die Gre des IP-Headers (20 bytes)
			Byte [] ReceiveBuffer = new Byte[256]; 

			nBytes = 0;
			nBytes = socket.ReceiveFrom(ReceiveBuffer, 256, 0, ref EndPointFrom);

			if (nBytes == SOCKET_ERROR) 
			{
				dwStop = SOCKET_ERROR;
			} 
			else 
			{
				dwStop = System.Environment.TickCount - dwStart; // stop timing
			}
		}

		socket.Close();     
		PingTime = (int)dwStop;
		return PingTime;
	}

	private static Int32 Serialize(  IcmpPacket packet, Byte [] Buffer, Int32 PacketSize, Int32 PingData )
	{
		Int32 cbReturn = 0;

		//
		// Die Struktur in das Array serialisieren.
		//
		int Index=0;

		Byte [] b_type = new Byte[1];
		b_type[0] = (packet.Type);

		Byte [] b_code = new Byte[1];
		b_code[0] = (packet.SubCode);

		Byte [] b_cksum = BitConverter.GetBytes(packet.CheckSum);
		Byte [] b_id = BitConverter.GetBytes(packet.Identifier);
		Byte [] b_seq = BitConverter.GetBytes(packet.SequenceNumber);
	    
		// Console.WriteLine("Serialize type ");
		Array.Copy( b_type, 0, Buffer, Index, b_type.Length );
		Index += b_type.Length;
	    
		// Console.WriteLine("Serialize code ");
		Array.Copy( b_code, 0, Buffer, Index, b_code.Length );
		Index += b_code.Length;

		// Console.WriteLine("Serialize cksum ");
		Array.Copy( b_cksum, 0, Buffer, Index, b_cksum.Length );
		Index += b_cksum.Length;

		// Console.WriteLine("Serialize id ");
		Array.Copy( b_id, 0, Buffer, Index, b_id.Length );
		Index += b_id.Length;

		Array.Copy( b_seq, 0, Buffer, Index, b_seq.Length );
		Index += b_seq.Length;

		// copy the data
	    
		Array.Copy( packet.Data, 0, Buffer, Index, PingData );

		Index += PingData;

		if( Index != PacketSize/* sizeof(IcmpPacket)  */) 
		{
			cbReturn = -1;
			return cbReturn;
		}

		cbReturn = Index;
		return cbReturn;
	}

	private static UInt16 checksum( UInt16[] buffer, int size )
	{
		Int32 cksum = 0;
		int counter;

		counter = 0;

		while ( size > 0 ) 
		{

			UInt16 val = buffer[counter];

			cksum += Convert.ToInt32( buffer[counter] );
			counter += 1;
			size -= 1;
		}

		cksum = (cksum >> 16) + (cksum & 0xffff);
		cksum += (cksum >> 16);
		return (UInt16)(~cksum);
	}
} 

public class IcmpPacket 
{ 
	public Byte  Type;    // Nachrichtentyp
	public Byte  SubCode;    // Subcode-Typ
	public UInt16 CheckSum;   // Einerkomplement-Prfsumme der Struktur
	public UInt16 Identifier;      // Bezeichner
	public UInt16 SequenceNumber;     // Sequenznummer
	public Byte [] Data;

} 