using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Threading;
using SharedComponent;

public class TcpServerTest
{
	private static void Main()
	{
		// Eine 1 MB groe Testdatei erstellen.
		FileStream fs = new FileStream("test.bin", FileMode.Create);
		fs.SetLength(100000);
		fs.Close();

		// Erstellen eines neuen Listeners (Abhrer) an Anschluss 8000.
		TcpListener listener = new TcpListener(8000);
        
		Console.WriteLine("Initialisiere Anschluss.");
		listener.Start();
        Console.WriteLine("Warte auf eingehende Verbindungsanforderungen...");
        
		int clientNum = 0;
		while (true)
		{
			try 
			{
				// Auf eine Verbindungsanforderung warten und einen fr
				// die Kommunikation initialisierten TcpClient zurckgeben. 
				TcpClient client = listener.AcceptTcpClient();
				Console.WriteLine("Server: Verbindung akzeptiert.");

				// Ein neues Objekt erstellen, um diese Verbindung zu bearbeiten.
				clientNum++;
				ClientHandler handler = new ClientHandler(client, "Client " +
					clientNum.ToString());

				// Dieses Objekt in einem anderen Thread ausfhren.
				Thread handlerThread = 
					new Thread(new ThreadStart(handler.Start));
				handlerThread.IsBackground = true;
				handlerThread.Start();

				// (Sie knnten Handler und HandlerThread auch zu einer Auflistung hinzufgen,
				// um die Client-Sitzungen zu berwachen.)
			}
			catch (Exception err) 
			{
				Console.WriteLine(err.ToString());
			}
		}
	}
}

public class ClientHandler 
{
	private TcpClient client;
	private string ID;

	// Die Menge, die in einen Block geschrieben wird (2 KB).
	private int bufferSize = 2048;

	// Der Puffer mit den Daten, die geschrieben werden sollen.
	private byte[] buffer;

	// Wird verwendet, um Daten aus einer Datei zu lesen.
	private FileStream fileStream;

	// Wird fr die Kommunikation mit dem Client verwendet.
	private NetworkStream networkStream;

	// Ein Signal zum Stoppen der Datenbermittlung.
	private bool fileStop = false;

	public ClientHandler(TcpClient client, string ID) 
	{
		this.buffer = new byte[bufferSize];
		this.client = client;
		this.ID = ID;
	}

	public void Start() 
	{
		// Den Netzwerk-Stream abrufen.
		networkStream = client.GetStream();

		// Objekte zum Senden und Empfangen von Text erstellen.
		BinaryWriter w = new BinaryWriter(networkStream);
		BinaryReader r = new BinaryReader(networkStream);

		if (r.ReadString() == ClientMessages.RequestConnect) 
		{
			w.Write(ServerMessages.AcknowledgeOK);
			Console.WriteLine(ID + ": Verbindung beendet.");

			string message = "";
			while (message != ClientMessages.Disconnect)
			{
				message = r.ReadString();
				if (message == ClientMessages.RequestData)
				{
					// Der Dateiname knnte vom Client zur Verfgung gestellt werden,
					// doch in diesem Beispiel wird die Testdatei direkt im Code angegeben.
					fileStream = new FileStream("test.bin", FileMode.Open);

					// Die Dateigre senden. (Auf diese Weise wei der Client, wie
					// viele Daten gelesen werden mssen).
					w.Write(fileStream.Length.ToString());

					// Diese Methode startet eine asynchrone Operation.
					StreamData(null);						
				}
			}
			fileStop = true;
			Console.WriteLine(ID + ": Verbindungstrennungsanforderung empfangen.");
		}
		else 
		{
			Console.WriteLine(ID + ": Verbindung konnte nicht beendet werden.");
		}

		// Bereinigung.
		client.Close();
		Console.WriteLine(ID + ": Client-Verbindung geschlossen.");

		Console.ReadLine();
	}


	private void StreamData(IAsyncResult asyncResult) 
	{
		// Abbrechen, wenn der Client getrennt wurde. 
		if (fileStop == true) {
			fileStop = false;
			return;
		}

		if (asyncResult != null)
		{
			// Ein Block wurde asynchron geschrieben.
			networkStream.EndWrite(asyncResult);
		}

		// Den nchsten Block aus der Datei abrufen.
		int bytesRead = fileStream.Read(buffer, 0, buffer.Length);

		// Wenn keine Bytes gelesen wurde, befindet sich der Stream am Ende der Datei.
		if (bytesRead > 0) 
		{
			Console.WriteLine("Neuen Block in den Stream schreiben.");

			// Den nchsten Block in den Netzwerk-Stream schreiben.
			networkStream.BeginWrite(buffer, 0, buffer.Length,
				new AsyncCallback(StreamData), null);
		}
		else 
		{
			// Operation beenden.
			Console.WriteLine("Streaming beendet.");
			fileStream.Close();
		}
	}

}