using System;
using System.Security.Policy;
using System.Security.Cryptography.X509Certificates;

public class AppDomainEvidenceExample {

    public static void Main() {
    
        // Eine neue Anwendungsdomne fr jeden Publisher erstellen,
        // dessen Code die Anwendung laden wird. Der CreateAppDomain-Methode
        // den Unternehmensnamen und den Namen einer Datei bergeben, die das
        // X.509v3-Zertifikat des Unternehmens enthlt.
        AppDomain appDom1 = CreateAppDomain("Litware", "litware.cer");
        AppDomain appDom2 = CreateAppDomain("Fabrikam", "fabrikam.cer");
        
        // Den Code der verschiedenen Publisher zur Ausfhrung in die
        // entsprechende Anwendungsdomne laden.
        // ...
    }

    // Eine Methode zur Erstellung einer neuen Anwendungsdomne, in die der
    // Code eines spezifischen Publishers zur Ausfhrung geladen wird. Das
    // Argument name gibt den Namen der Anwendungsdomne an. Das Argument
    // certFile legt den Namen einer Datei fest, die das X.509v3-Zertifikat des
    // Software-Publishers enthlt, dessen Code in der neuen Anwendungsdomne
    // ausgefhrt wird.
    private static AppDomain CreateAppDomain(string name, string certFile){
        
        // Ein neues X509Certificate-Objekt aus dem X.509v3-Zertifikat
        // erstellen, das in der angegebenen Datei enthalten ist.
        X509Certificate cert = 
            X509Certificate.CreateFromCertFile(certFile);
        
        // Einen neuen Publisher-Beweis aus dem X509Certificate-Objekt erstellen.
        Publisher publisherEvidence = new Publisher(cert);

        // Eine neue Evidence-Auflistung erstellen.
        Evidence evidence = new Evidence();

        // Der Evidence-Auflistung den Publisher-Beweis hinzufgen.
        evidence.AddHost(publisherEvidence);

        // Eine neue Anwendungsdomne mit der Evidence-Auflistung
        // erstellen, die den Publisher-Beweis enthlt, und die neu
        // erstellte Anwendungsdomne zurckgeben.
        return AppDomain.CreateDomain(name, evidence);
    }
}
