using System;
using System.IO;
using System.Security.Principal;
using System.Security.Permissions;
using System.Runtime.InteropServices;

// Sicherstellen, dass die Assembly ber die Berechtigung verfgt, nicht
// verwalteten Code auszufhren und den Thread-Prinzipal zu kontrollieren.
[assembly:SecurityPermission(SecurityAction.RequestMinimum,
    UnmanagedCode=true, ControlPrincipal=true)]

public class ImpersonationExample {

    // Einige Konstanten fr die LogonUser-Funktion definieren.
    const int LOGON32_PROVIDER_DEFAULT = 0;
    const int LOGON32_LOGON_INTERACTIVE = 2;

    // Die Win32-Funktion LogonUser aus advapi32.dll importieren.
    // "SetLastError = true" setzen, um den korrekten Zugriff auf
    // die Win32-Fehlercodes zu gewhrleisten.
    [DllImport("advapi32.dll", SetLastError=true)]
    static extern int LogonUser(string userName, string domain, 
        string password, int logonType, int logonProvider,
        ref IntPtr accessToken);

    public static void Main(string[] args) {

        // Eine neue IntPtr-Struktur erstellen, die das von der
        // LogonUser-Funktion zurckgegebene Zugriffstoken speichert.
        IntPtr accessToken = IntPtr.Zero;
        
        // LogonUser aufrufen, um ein Zugriffstoken fr den angegebenen Benutzer
        // abzurufen. Die Variable accessToken wird als Verweis an LogonUser
        // bergeben und referenziert das Windows-Zugriffstoken, wenn
        // LogonUser erfolgreich ist.
        int result = LogonUser(
            args[0],                    // Name des anzumeldenden Benutzers.
            ".",                        // Die lokale Kontendatenbank verwenden.
            args[1],                    // Kennwort des Benutzers.
            LOGON32_LOGON_INTERACTIVE,  // Interaktive Anmeldung erstellen.
            LOGON32_PROVIDER_DEFAULT,   // Den Standard-Anmelde-Provider verwenden.
            ref accessToken             // Nimmt Zugriffstoken-Handle entgegen.
        );
        
        // Wenn der Rckgabecode von LogonUser 0 ist, ist ein Fehler aufgetreten.
        // Fehler anzeigen und Routine verlassen.
        if (result == 0)  {        	

            Console.WriteLine("LogonUser gab folgenden Fehler zurck: {0}", 
                Marshal.GetLastWin32Error());
            
        } else {

            // Eine neue WindowsIdentity aus dem Windows-Zugriffstoken erstellen.
            WindowsIdentity identity = new WindowsIdentity(accessToken);
    
            // Die aktive Identitt anzeigen.
            Console.WriteLine("Identitt vor Personifikation = {0}",
                WindowsIdentity.GetCurrent().Name);

            // Identitt des angegebenen Benutzers annehmen, und einen
            // Verweis auf das zurckgegebene WindowsImpersonationContext-
            //Objekt speichern, das die Informationen enthlt, die
            // erforderlich sind, um zum ursprnglichen Benutzerkontext
            // zurckzukehren.
            WindowsImpersonationContext impContext = 
                identity.Impersonate();

            // Die aktive Identitt anzeigen.
            Console.WriteLine("Identity during impersonation = {0}",
                WindowsIdentity.GetCurrent().Name);

            // *****************************************        
            // Aktionen im Namen des Benutzers ausfhren,
            // dessen Identitt angenommen wurde.
            // ***************************************** 
            
            // Mithilfe des WindowsImpersonationContext-Objekt zum
            // ursprnglichen Windows-Benutzer zurckkehren.
            impContext.Undo();

            // Die aktive Identitt anzeigen.
            Console.WriteLine("Identitt nach Personifikation = {0}",
                WindowsIdentity.GetCurrent().Name);
        }
		// Auf Tastendruck warten.
		Console.ReadLine();
	}
}
