using System;
using System.Text;
using System.Security.Cryptography;

public class VerifyHashExample {

    public static void Main(string[] args) {
    
        // Den HashAlgorithmus erstellen, der im ersten 
        // Argument angegeben ist.
        using (HashAlgorithm hashAlg = HashAlgorithm.Create(args[0])) {

            // Ein Byte-Array aus dem Klartext erstellen, der im zweiten
            // Argument angegeben ist.
            byte[] plainttext = Encoding.Default.GetBytes(args[1]);
                
            // Den Hashcode aus dem angegebenen Klartext generieren.
            byte[] hash = hashAlg.ComputeHash(plainttext);

            // berprfen, ob der neue Hashcode mit dem angegebenen Hashcode bereinstimmt, der 
            // im dritten Argument enthalten ist. Sowohl das Hex- als auch das Base64-Verfahren testen.
            Console.WriteLine("berprftes Hex-Hash = {0}", VerifyHexHash(hash, args[2]));
            Console.WriteLine("berprftes Base64-Hash = {0}", VerifyB64Hash(hash, args[2]));
        }
    
        // Auf Tastendruck warten.
        Console.ReadLine();
    }
    
    // Eine Methode, um einen neu generierten Hashcode mit einem 
    // bereits vorhandenen Hashcode zu vergleichen, der von einer Hex-Code-Zeichenfolge reprsentiert wird.
    private static bool VerifyHexHash(byte[] hash, string oldHashString) {
        
        // Eine String-Darstellung der Hashcode-Bytes erstellen.
        System.Text.StringBuilder newHashString = 
            new System.Text.StringBuilder(hash.Length);
        
        // Jedes Byte als grogeschriebenen zweistelligen Hex-String anfgen.
        foreach (byte b in hash) {
            newHashString.AppendFormat("{0:X2}", b);
        }
        
        // Die String-Darstellungen des alten und neuen Hashcodes 
        // vergleichen und das Ergebnis zurckgeben.
        return (oldHashString == newHashString.ToString());
    }

    // Eine Methode, um einen neu generierten Hashcode mit einem
    // bereits vorhandenen Hashcode zu vergleichen, der von einer Base64-kodierten Zeichenfolge reprsentiert wird.
    private static bool VerifyB64Hash(byte[] hash, string oldHashString) {
        
        // Eine Base64-Darstellung der Hashcode-Bytes erstellen.
        string newHashString = System.Convert.ToBase64String(hash); 
        
        // Die String-Darstellungen des alten und neuen Hashcodes 
        // vergleichen und das Ergebnis zurckgeben.
        return (oldHashString == newHashString);
    }
    
    // Eine Methode, um einen neu generierten Hashcode mit einem
    // bereits vorhandenen Hashcode zu vergleichen, der von einem Byte-Array reprsentiert wird.
    private static bool VerifyByteHash(byte[] hash, byte[] oldHash) {
        
        // Wenn eines der Arrays null ist oder die Arrays von unterschiedlicher Lnge sind,
        // stimmen sie nicht berein.
        if (hash == null || oldHash == null || 
            hash.Length != oldHash.Length) return false;
            
        // Die Byte-Arrays durchlaufen und alle Byte-Werte miteinander vergleichen.
        for (int count = 0; count < hash.Length; count++) {
            if (hash[count] != oldHash[count]) return false;
        }
    
        // Die Hashcodes stimmen berein.
        return true;
    }
}
