public class StoreAsymmetricKeyExample {

    public static void Main(string[] args) {
        
        switch (args[0].ToUpper()) {

            case "LOAD":
                // Die LoadKeys-Methode aufrufen und den Namen des
                // Schlsselcontainers bergeben.
                LoadKeys(args[1]);
                break;
                                
            case "DEL":
                // Die DeleteKeys-Methode aufrufen und den Namen des
                // Schlsselcontainers bergeben.
                DeleteKeys(args[1]);
                break;
        }

        // Auf Tastendruck warten.
        System.Console.ReadLine();
    }

    // Eine Methode zur Erstellung eines RSACryptoServiceProvider-Objekts und zum Laden
    // der Schlssel aus einem benannten CryptoAPI-Schlsselcontainer, sofern die Schlssel
    // vorhanden sind. Andernfalls generiert das RSACryptoServiceProvider-Objekt automatisch
    // neue Schlssel und speichert diese im benannten Container.
    public static void LoadKeys(string container) {

        // Ein neues CspParameters-Objekt erstellen und dessen KeyContainerName-
        // Feld auf den Namen des angegebenen Containers setzen.
        System.Security.Cryptography.CspParameters cspParams = 
            new System.Security.Cryptography.CspParameters();
        cspParams.KeyContainerName = container;
        
        // Einen neuen asymmetrischen RSA-Algorithmus erstellen und das CspParameters-
        // Objekt bergeben, das die Schlsselcontainerdetails festlegt.
        using (System.Security.Cryptography.RSACryptoServiceProvider rsaAlg = 
         new System.Security.Cryptography.RSACryptoServiceProvider(cspParams)){

            // Das RSACryptoServiceProvider-Objekt konfigurieren, sodass es die Schlssel
            // im Schlsselcontainer speichert.
            rsaAlg.PersistKeyInCsp = true;

            // Die ffentlichen Schlssel im Konsolenfenster anzeigen.
            System.Console.WriteLine(rsaAlg.ToXmlString(false));
            
            // Da das RSACryptoServiceProvider-Objekt so konfiguriert wurde, dass es die Schlssel speichert,
            // werden diese im angegebenen Schlsselcontainer gespeichert.
        }
    }
    
    // Eine Methode zur Erstellung eines RSACryptoServiceProvider-Objekts und zum Lschen bereits vorhandener
    // Schlssel aus dem benannten CryptoAPI-Schlsselcontainer.
    // 
    public static void DeleteKeys(string container) {
        
        // Ein neues CspParameters-Objekt erstellen und dessen KeyContainerName-Feld
        // auf den Namen des zu bereinigenden Containers setzen.
        System.Security.Cryptography.CspParameters cspParams = 
            new System.Security.Cryptography.CspParameters();
        cspParams.KeyContainerName = container;
        
        // Einen neuen asymmetrischen RSA-Algorithmus erstellen und das CspParameters-
        // Objekt bergeben, das die Schlsselcontainerdetails festlegt.
        using (System.Security.Cryptography.RSACryptoServiceProvider rsaAlg = 
         new System.Security.Cryptography.RSACryptoServiceProvider(cspParams)){

            // Das RSACryptoServiceProvider-Objekt konfigurieren, sodass es die Schlssel
            // nicht im Schlsselcontainer speichert.

            rsaAlg.PersistKeyInCsp = false;

            // Die ffentlichen Schlssel im Konsolenfesnter anzeigen. Da wir nach diesem
            // Aufruf Dispose() aufrufen, scheinen sich die vorhandenen Schlssel erst
            // beim zweiten Aufruf der Methode zu ndern.
            System.Console.WriteLine(rsaAlg.ToXmlString(false));

            // Wenn der Code diesen "using"-Block verlsst, wird die Dispose-Methode des
            // RSACryptoServiceProvider-Objekts aufgerufen. Da das Objekt so konfiguriert wurde, dass
            // es die Schlssel NICHT speichert, wird der betroffene Container gelscht.
            // Wrde anstelle von Dispose() die Methode rsaAlg.Clear() aufgerufen, htte dies
            // denselben Effekt, da Dispose() in diesem Fall indirekt aufgerufen wrde.
        }
    }
}
