using System;
using System.Text;
using System.Collections;

public class Employee : ICloneable {
    
    public string Name;
    public string Title;

    // Einfacher Employee-Konstruktor
    public Employee(string name, string title) {

        Name = name;
        Title = title;
    }
        
    // Einen Klon mit der Methode Object.MemberwiseClone erstellen, da die
    // die Klasse Employee nur String-Verweise enthlt
    public object Clone() {
        
        return MemberwiseClone();
    }

    // Gibt eine Zeichenfolgendarstellung des Employee-Objekts zurck
    public override string ToString() {
        
        return string.Format("{0} ({1})", Name, Title);
    }        
}

public class Team : ICloneable {
    
    public ArrayList TeamMembers = new ArrayList();
    
    public Team() {
    }

    // Privater Konstruktor, der von der Clone-Methode aufgerufen wird,
    // um ein neues Team-Objekt zu erstellen und dessen ArrayList mit
    // geklonten Employee-Objekten aus einer bereitgestellten ArrayList
    // zu fllen.
    private Team(ArrayList members) {
        
        foreach (Employee e in members) {
            
            TeamMembers.Add(e.Clone());
        }
    }
    
    // Dem Team ein Employee-Objekt hinzufgen
    public void AddMember(Employee member) {
    
        TeamMembers.Add(member);
    }
    
    // Object.ToString berschreiben, um eine Zeichenfolgendarstellung
    // des Team zurckzugeben.
    public override string ToString() {
                
        StringBuilder str = new StringBuilder();
        
        foreach (Employee e in TeamMembers) {
            
            str.AppendFormat("{0}\n\r", e);
        }
        
        return str.ToString();
    }        
    
    public object Clone() {
        
        // Eine tiefe Kopie des Teams erstellen, indem der private Team-
        // Konstruktor aufgerufen und die ArrayList bergeben wird, die
        // die Team-Mitglieder enthlt.
        return new Team(this.TeamMembers);
        
        // Der folgende Befehl wrde eine flache Kopie des Teams erstellen.
        // return MemberwiseClone();
    }
        
    public static void Main() {
        
        // Das Original-Team erstellen
        Team team = new Team();
        team.AddMember(new Employee("Frank", "Entwickler"));
        team.AddMember(new Employee("Kathy", "Tester"));
        team.AddMember(new Employee("Chris", "Support"));
        
        // Das Original-Team klonen
        Team clone = (Team)team.Clone();

        // Das Original-Team anzeigen
        Console.WriteLine("Original-Team:");
        Console.WriteLine(team);

        // Das geklonte Team anzeigen
        Console.WriteLine("Geklontes Team:");
        Console.WriteLine(clone);
        
        // Eine nderung vornehmen
        Console.WriteLine("*** nderung am Original-Team vornehmen ***\n\r");
        ((Employee)team.TeamMembers[0]).Name = "Luke";
        ((Employee)team.TeamMembers[0]).Title = "Manager";

        // Original-Team anzeigen
        Console.WriteLine("Original-Team:");
        Console.WriteLine(team);
        
        // Geklontes Team anzeigen
        Console.WriteLine("Geklontes Team:");
        Console.WriteLine(clone);

        Console.ReadLine();
    }
}
     

    

