using System;

public class Person : IFormattable {

    // Private Member, die den Titel und den Namen der Person speichern.
    private string title;
    private string[] names;

    // Konstruktor zum Setzen des Titels und des Namens der Person.
    public Person(string title, params string[] names) {

        this.title = title;
        this.names = names;
    }

    // Die Methode Object.ToString berschreiben, um den Namen der
    // Person im allgemeinen Format zurckzugeben.
    public override string ToString() {
        return ToString("G", null);
    }

    // Implementierung der Methode IFormattable.ToString, um den
    // Namen der Person in einem anderen Format zurckzugeben, das
    // auf der bereitgestellten Formatzeichenfolge basiert.
    public string ToString(string format, IFormatProvider formatProvider) {

        string result = null;

        // Das allgemeine Format verwenden, wenn kein Format angegeben wurde.
        if (format == null) format = "G";

        // Die Inhalte der Formatzeichenfolge bestimmen das Format des
        // zurckgegebenen Namens.
        switch (format.ToUpper()[0]) {

            case 'S':
                // Kurzform verwenden - Erster Buchstabe des Vornamens und Nachname.
                result = names[0][0] + ". " + names[names.Length-1];
                break;

            case 'P':
                // Hflichkeitsform verwenden - Titel, abgekrzter
                // Vorname und Nachname.
                // Den Titel der Person zum Ergebnis hinzufgen.
                if (title != null && title.Length != 0) {
                    result = title + ". ";
                }
                // Den abgekrzten Vornamen und den Nachnamen hinzufgen.
                for (int count = 0; count < names.Length; count++) {
                    
                    if ( count != (names.Length - 1)) {
                        result += names[count][0] + ". ";
                    } else {
                        result += names[count];
                    }
                }
                break;

            case 'I':
                // Informelle Form verwenden - nur Vorname.
                result = names[0];
                break;

            case 'G':
            default:
                // Allgemeine Standardform verwenden - Vorname und Nachname.
                result = names[0] + " " + names[names.Length-1];
                break;
        }
        return result;
    }

    public static void Main() {

        // Ein Person-Objekt erstellen, das einen Mann namens
        // Mr. Richard Glen David Peters reprsentiert.
        Person person = 
            new Person("Mr", "Richard", "Glen", "David", "Peters");

        // Den Namen der Person in verschiedenen Formaten anzeigen.
        System.Console.WriteLine("Sehr geehrter {0:G},", person);
        System.Console.WriteLine("Sehr geehrter {0:P},", person);
        System.Console.WriteLine("Sehr geehrter {0:I},", person);
        System.Console.WriteLine("Sehr geehrter {0},", person);
        System.Console.WriteLine("Sehr geehrter {0:S},", person);

        Console.ReadLine();
    }
}
