public class SingletonExample {

    // Ein statischer Member, der einen Verweis auf die Singleton-Instanz speichert.
    private static SingletonExample instance;

    // Ein statischer Konstruktor zur Erstellung der Singleton-Instanz.
    // Eine Alternative hierzu besteht darin, mit der Eigenschaft Instance
    // die verzgerte Initialisierung zu verwenden.
    static SingletonExample () {
        instance = new SingletonExample();
    }
        
    // Ein privater Konstruktor, der verhindert, dass der Code zustzliche
    // Instanzen des Singletion-Typs generiert.
    private SingletonExample () {}
    
    // Eine ffentliche Eigenschaft fr den Zugriff auf die Singletion-Instanz.
    public static SingletonExample Instance { 
        
        get { return instance; }
    }
    
    // ffentliche Methoden, die die Singleton-Funktionalitt zur Verfgung stellen.
    public void SomeMethod1 () { /*..*/ }
    public void SomeMethod2 () { /*..*/ }
    
    public static void Main() {
        
        // Verweise auf Singletion abrufen und Methode aufrufen.
        SingletonExample s = SingletonExample.Instance;
        s.SomeMethod1();
        
        // Singletion-Funktionalitt ohne Verweis ausfhren.
        SingletonExample.Instance.SomeMethod2();
    }
}
