using System;
using Microsoft.Win32;

public class RegistryExample {

    public static void Main(String[] args) {	
		
        if (args.Length > 0) {

            // Den Basisschlssel CurrentUser ffnen.
            using(RegistryKey root = Registry.CurrentUser) {

                // Den Verwendungszhler aktualisieren.
                UpdateUsageCounter(root);

                // Die Windows-Registrierung rekursiv nach jedem Schlssel
                // mit dem angegebenen Namen durchsuchen. 
                SearchSubKeys(root, args[0]);
            }
        }				

        // Auf Tastendruck warten.
        Console.WriteLine("Main-Methode beendet. Drcken Sie Enter.");
        Console.ReadLine();
    }
	
    public static void UpdateUsageCounter(RegistryKey root) {

        // Den Schlssel erstellen, der den Verwendungszhler speichert,
        // oder einen Verweis auf einen bereits vorhandenen Schlssel abrufen.
        RegistryKey countKey = root.CreateSubKey("RegistryExample");

        // Den aktuellen Verwendungszhlerwert auslesen und dem Standardwert 0
        // (Null) festlegen. Das Objekt in ein Int32 konvertieren und einem int zuweisen.
        int count = (Int32)countKey.GetValue("UsageCount", 0);

        // Den erhhten Verwendungszhler wieder in der Windows-Registrierung
        // speichern oder einen neuen Wert generieren, wenn der Zhler noch nicht vorhanden ist.
        countKey.SetValue("UsageCount", ++count);
    }

    public static void SearchSubKeys(RegistryKey root, String searchKey) {

        // Alle Unterschlssel des aktuellen Schlssels in einer Schleife durchlaufen.
        foreach (string keyname in root.GetSubKeyNames()) {

            try {				
                using (RegistryKey key = root.OpenSubKey(keyname)) {	
                    if (keyname == searchKey) PrintKeyValues(key);
                    SearchSubKeys(key, searchKey);
                }
            } catch (System.Security.SecurityException) {
                // Fr dieses Beispiel wird SecurityException ignoriert.
                // Einige Unterschlssel von HKEY_CURRENT_USER sind gesichert und
                // lsen die Ausnahme SecurityException aus, wenn sie geffnet werden.
            }
        }											
    }		
		
    public static void PrintKeyValues(RegistryKey key) {	
	
        // Den Namen des bereinstimmenden Unterschlssels sowie die Anzahl
        // der darin enthaltenen Werte anzeigen.
        Console.WriteLine("Registrierungsschlssel gefunden: {0} enthlt {1} Werte",
            key.Name, key.ValueCount);

        // Die Werte in einer Schleife durchlaufen und anzeigen. 
        foreach (string valuename in key.GetValueNames()) {			
            if (key.GetValue(valuename) is String) {			
                Console.WriteLine("  Wert: {0} = {1}", 
                    valuename, key.GetValue(valuename));
            }
        }
    } 				
}
