using System;
using System.Timers;
using System.ServiceProcess;

public class ServiceExample : ServiceBase {
	
    // Ein Timer, der kontrolliert, wann ServiceExample in das
    // Ereignisprotokoll schreibt.
    private System.Timers.Timer timer;

    public ServiceExample() {

        // Die Eigenschaft ServiceBase.ServiceName setzen.
        ServiceName = "ServiceExample";

        // Die Steuerungsnachrichten konfigurieren, die der Dienst bearbeiten wird.
        CanStop = true;
        CanPauseAndContinue = true;

        // Den Dienst so konfigurieren, dass er wichtige Ereignisse im
        // Anwendungsereignisprotokoll protokolliert.
        AutoLog = true;
    }
	
    // Die Methode, die ausgefhrt wird, wenn der Timer abgelaufen
    // ist - schreibt einen Eintrag in das Anwendungsereignisprotokoll.
    private void WriteLogEntry(object sender, ElapsedEventArgs e) {

        // Das von der Klasse ServiceBase automatisch konfigurierte
        // EventLog-Objekt verwenden, um in das Ereignisprotokoll zu schreiben.
        EventLog.WriteEntry("ServiceExample aktiv: " + e.SignalTime);
    }					

    protected override void OnStart(string[] args) {

        // Aus dem args-Array den Zeitabstand zwischen den Protokolleintragungen
        // extrahieren. Standardmig 5000 Millisekunden verwenden und ein
        // Minimum von 1000 Millisekunden erzwingen.
        double interval;

        try {
            interval = System.Double.Parse(args[0]);
            interval = Math.Max(1000, interval);
        } catch {
            interval = 5000;
        }

        EventLog.WriteEntry(String.Format("ServiceExample wurde gestartet. " +
            "Schreibe Protokolleintrge alle {0} Millisekunden...", interval));

        // System.Timers.Timer erstellen, konfigurieren und starten, um die
        // Methode WriteLogEntry regelmig aufzurufen. Die Methoden Start und
        // Stop der Klasse System.Timers.Timer vereinfachen das Starten,
        // Anhalten, Fortsetzen und Stoppen des Dienstes.
        timer = new Timer();
        timer.Interval = interval;
        timer.AutoReset = true;
        timer.Elapsed += new ElapsedEventHandler(WriteLogEntry);
        timer.Start();
    }
	
    protected override void OnStop() {

        EventLog.WriteEntry("ServiceExample wird gestoppt...");
        timer.Stop();

        // Freie Systemresourcen, die vom Timer-Objekt verwendet werden.
        timer.Dispose();
        timer = null;
    }
		
    protected override void OnPause() {

        if (timer != null) {
            EventLog.WriteEntry("ServiceExample wird angehalten...");
            timer.Stop();
        }
    }
			
    protected override void OnContinue() {

        if (timer != null) {
            EventLog.WriteEntry("ServiceExample wird fortgesetzt...");
            timer.Start();
        }
    }
		
    public static void Main() {

        // Eine Instanz der Klasse ServiceExample erstellen, die einen Eintrag
        // in das Anwendungsereignisprotokoll schreibt. Das Objekt der
        // statischen Methode ServiceBase.Run bergeben.
        ServiceBase.Run(new ServiceExample());
    }	
}
